; dungeon_map_maker.scm
; by Rob Antonishen
; http://ffaat.pointclark.net

; Version 1.6 (20100602)

; Description
;
; makes a more "artistic" dungeon map using monochrome template
;
; Changes
; 1.1 - added online fetch of a random map from isomage using the syntax:
; 1.2 - removed online fetch because of lockups when not available, added 
;       invert colour selection.
; 1.3 - added an optional selection to specify a mask to the displace operation 
;       and allow only part of the map to be "rugged".  This only works in the
;       basic mode, not the batch mode.
;     - moved the batch mode to a "Batch Tools" submenu.
; 1.4 - added option for hex grid.  Only works if hex script is installed
;       hex grid script is available at http://registry.gimp.org/node/15852
; 1.5 - added grid option "none".
;     - bug fix for greyscale input images
;     - changed wall shadows to be a transparent layer rather than a multiplied layer
;     - added floor option - no floor (transparent)
; 1.6 - added wall fadeout - black (vtt), white (printing), or transparent

; License:
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version. 
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; The GNU Public License is available at
; http://www.gnu.org/copyleft/gpl.html

(define (script_fu_dungeon_map_maker img inLayer inGridSize inGridType inSelectColour inSelectThresh inSelectWhat inDistort inDistMask inLightAngle
                                     inFloorMode inFloorPattern inFloorNoise inGridColour
                                     inWallMode inWallPattern inWallStyle inWallFade
                                     inStrokeStyle inStrokeBevel inStrokeSize inStrokePos inStrokeColour inStrokePattern)
  (let*
    (
      (width 0)
      (height 0)
      (varLayerName "")
      (layerFloor 0)
      (layerFloorOverlay 0)
      (layerWall 0)
      (maskWall 0)
      (layerGrid 0)
      (layerGridBump 0)
      (layerWallFade 0)
      (layerWallStyle 0)  
      (layerWallShadow 0)
      (selInside 0)
      (layerTemp1 0)
      (layerTemp2 0)
      (url "")
      (check 0)
      (handler (car (gimp-message-get-handler)))
      (origlayer 0)
    )
    ;  it begins here
    ;(begin (gimp-message (string-append "check-" (number->string check))) (set! check (+ check 1)))
    
  
    
    (set! width (car (gimp-image-width img)))
    (set! height (car (gimp-image-height img)))
    (set! varLayerName (car (gimp-drawable-get-name inLayer)))
      
    (gimp-context-push)
    (gimp-image-undo-group-start img)
    
    ;logging
    ;(gimp-message-set-handler ERROR-CONSOLE)
    ;(gimp-message-set-handler CONSOLE)
    ;(gimp-message-set-handler MESSAGE-BOX)
    ;or start GIMP wwith "gimp --console-messages" to spawn a console box
    ;then use this:
    ;(gimp-message "foobar") 

    ;testing for functions defined
    ;(if (defined? 'plug-in-shift) (gimp-message "It Exists") (gimp-message "Doesnt Exist"))

    ;check for hex option and no script
    (when (and (or (= inGridType 1) (= inGridType 2)) (not (defined? 'script-fu-hex_grid)))
      (gimp-message-set-handler MESSAGE-BOX)
      (gimp-message "Hex Grid Script must be installed.\nIt can be downloaded from\nhttp://registry.gimp.org/node/15852")
      (gimp-message-set-handler handler)
      (gimp-image-undo-group-end img)
      (gimp-progress-end)
      (gimp-context-pop)
      (quit)
    )
     
    (gimp-selection-none img)

    ;ensure image is in RGB
    (if (not (equal? (car (gimp-image-base-type img)) RGB)) (gimp-image-convert-rgb img))
    
    ;Enlarge image horizontally
    (set! layerTemp1 (car (gimp-layer-copy inLayer TRUE)))
    (gimp-image-add-layer img layerTemp1 -1)    
    (gimp-layer-resize layerTemp1 1 height 0 0) 

    (set! layerTemp2 (car (gimp-layer-copy inLayer TRUE)))
    (gimp-image-add-layer img layerTemp2 -1)    
    (gimp-layer-resize layerTemp2 1 height (- 1 width) 0) 
    
    (gimp-image-resize img (+ width inGridSize inGridSize) height inGridSize 0)
    (set! layerTemp1 (car (gimp-drawable-transform-scale-default layerTemp1 0 0 inGridSize height FALSE TRANSFORM-RESIZE-ADJUST)))
    (set! layerTemp2 (car (gimp-drawable-transform-scale-default layerTemp2 
                           (+ width inGridSize) 0 (+ width (* 2 inGridSize)) height FALSE TRANSFORM-RESIZE-ADJUST)))

    (set! layerTemp1 (car (gimp-image-merge-down img layerTemp2 EXPAND-AS-NECESSARY)))
    (set! inLayer (car (gimp-image-merge-down img layerTemp1 EXPAND-AS-NECESSARY)))

    (set! width (car (gimp-image-width img)))
    
    ;Enlarge image vertically
    (set! layerTemp1 (car (gimp-layer-copy inLayer TRUE)))
    (gimp-image-add-layer img layerTemp1 -1)    
    (gimp-layer-resize layerTemp1 width 1 0 0) 

    (set! layerTemp2 (car (gimp-layer-copy inLayer TRUE)))
    (gimp-image-add-layer img layerTemp2 -1)    
    (gimp-layer-resize layerTemp2 width 1 0 (- 1 height)) 
    
    (gimp-image-resize img width (+ height inGridSize inGridSize) 0 inGridSize)
    (set! layerTemp1 (car (gimp-drawable-transform-scale-default layerTemp1 0 0 width inGridSize FALSE TRANSFORM-RESIZE-ADJUST)))
    (set! layerTemp2 (car (gimp-drawable-transform-scale-default layerTemp2 
                           0 (+ height inGridSize) width (+ height (* 2 inGridSize)) FALSE TRANSFORM-RESIZE-ADJUST)))

    (set! layerTemp1 (car (gimp-image-merge-down img layerTemp2 EXPAND-AS-NECESSARY)))
    (set! inLayer (car (gimp-image-merge-down img layerTemp1 EXPAND-AS-NECESSARY)))
    
    (set! height (car (gimp-image-height img))) 

    ;to hide layer later
    (set! origlayer inLayer)
    
    ;If distortion
    (if (> inDistort 0)
      (begin
        (if (not (equal? inDistMask -1))
           (if (equal? (car (gimp-drawable-get-image inDistMask)) img)
              (gimp-selection-load inDistMask)))
            (set! inLayer (car (gimp-layer-new-from-drawable inLayer img)))
        (gimp-drawable-set-name inLayer (string-append varLayerName " distorted"))
        (gimp-image-add-layer img inLayer -1)   
        (gimp-layer-add-alpha inLayer)
        (plug-in-shift RUN-NONINTERACTIVE img inLayer (trunc (/ (* inDistort inGridSize) 100)) 0)
        (plug-in-shift RUN-NONINTERACTIVE img inLayer (trunc (/ (* inDistort inGridSize) 100)) 1)
        (plug-in-gauss RUN-NONINTERACTIVE img inLayer (/ inGridSize 5) (/ inGridSize 5) 1)
        (gimp-levels inLayer HISTOGRAM-VALUE 118 138 1 0 255)
        (if (not (equal? inDistMask -1))
           (if (equal? (car (gimp-drawable-get-image inDistMask)) img)
              (gimp-selection-none img)))
      )
    )  
      
      
    ;select by colour
    (gimp-by-color-select inLayer inSelectColour inSelectThresh CHANNEL-OP-REPLACE TRUE FALSE 0 FALSE)
    
    ;invert if walls
    (if (= inSelectWhat 1) (gimp-selection-invert img))
    
    (set! selInside (car (gimp-selection-save img)))
    (gimp-drawable-set-name selInside "Dungeon Mask")
    (gimp-selection-none img)
    
    (unless (equal? inFloorMode 2) ; skip if no floor chosen
      ;create floor layer
      (set! layerFloor (car (gimp-layer-new-from-drawable inLayer img)))
      (gimp-drawable-set-name layerFloor (string-append varLayerName " floor"))
      (gimp-image-add-layer img layerFloor -1)    
      (gimp-layer-add-alpha layerFloor)
    
      (if (= inFloorMode 0)
        (begin   ;render floor
          (plug-in-plasma RUN-NONINTERACTIVE img layerFloor (rand 32768) 7)
          (gimp-desaturate layerFloor)
          (gimp-levels layerFloor HISTOGRAM-VALUE 0 255 1 108 148)
          (plug-in-bump-map RUN-NONINTERACTIVE img layerFloor layerFloor inLightAngle 50 30 0 0 0 0 TRUE TRUE 0)
        )
        (begin   ;use pattern
          (gimp-context-set-pattern inFloorPattern)
          (gimp-drawable-fill layerFloor 4) ;pattern fill = 4
        )
      )
    
      ;floor noise overlay
      (if (= inFloorNoise TRUE)
        (begin
          (set! layerFloorOverlay (car (gimp-layer-new-from-drawable inLayer img)))
          (gimp-drawable-set-name layerFloorOverlay (string-append varLayerName " floor noise"))
          (gimp-image-add-layer img layerFloorOverlay -1) 
          (gimp-layer-add-alpha layerFloorOverlay)
          (plug-in-solid-noise RUN-NONINTERACTIVE img layerFloorOverlay FALSE FALSE (rand 32768) 4 4 4)
          (gimp-layer-set-mode layerFloorOverlay OVERLAY-MODE)
        )
      )
    ) ; end of floor check!

    (unless (equal? inGridType 3) ;start of grid only if grid chosen 3 is no grid
      ;draw grid - black on transparent
      (set! layerGrid (car (gimp-layer-new-from-drawable inLayer img)))
      (gimp-drawable-set-name layerGrid (string-append varLayerName " grid"))
      (gimp-image-add-layer img layerGrid -1) 
      (gimp-layer-add-alpha layerGrid)
      (gimp-selection-all img)
      (gimp-edit-clear layerGrid)
      (gimp-selection-none img)
    
      (cond 
        (( equal? inGridType 0 ) ;square
           (plug-in-grid RUN-NONINTERACTIVE img layerGrid (max 1 (/ inGridSize 50)) inGridSize 0 (list 0 0 0) 255 
                                                          (max 1 (/ inGridSize 50)) inGridSize 0 (list 0 0 0) 255 
                                                          0 0 0 (list 0 0 0) 255)
        )
        (( equal? inGridType 1 ) ;horizontal hex
          (script-fu-hex_grid img layerGrid 2 inGridSize 0 (max 1 (/ inGridSize 50)) inGridSize 0 (list 0 0 0))  
        )
        (( equal? inGridType 2 ) ;vertical hex
          (script-fu-hex_grid img layerGrid 2 inGridSize 1 (max 1 (/ inGridSize 50)) 0 inGridSize (list 0 0 0))  
        )
      )

      ;create blured copy for bumpmap
      (set! layerTemp1 (car (gimp-layer-new-from-drawable layerGrid img)))
      (gimp-image-add-layer img layerTemp1 -1)    
      (gimp-context-set-background (list 255 255 255))
      (gimp-layer-flatten layerTemp1)
      (plug-in-gauss RUN-NONINTERACTIVE img layerTemp1 (/ inGridSize 10) (/ inGridSize 10) 1)

      ;bumpmap grid
      (set! layerGridBump (car (gimp-layer-new-from-drawable layerGrid img)))
      (gimp-drawable-set-name layerGridBump (string-append varLayerName " grid bump"))
      (gimp-image-add-layer img layerGridBump -1) 
      (gimp-context-set-foreground (list 127 127 127))
      (gimp-drawable-fill layerGridBump FOREGROUND-FILL)
      (plug-in-bump-map RUN-NONINTERACTIVE img layerGridBump layerTemp1 inLightAngle 10 30 0 0 0 0 TRUE FALSE 1)
      (gimp-layer-set-mode layerGridBump OVERLAY-MODE)
    
      ;remove blured layer
      (gimp-image-remove-layer img layerTemp1)
    
      ;set grid colour as desired
      (cond 
        (( equal? inGridType 0 ) ;square
          (plug-in-grid RUN-NONINTERACTIVE img layerGrid 1 inGridSize 0 inGridColour 255 1 inGridSize 0 inGridColour 255 0 0 0 inGridColour 255)
        )
        (( equal? inGridType 1 ) ;horizontal hex
          (script-fu-hex_grid img layerGrid 2 inGridSize 0 1 inGridSize 0 inGridColour)
        )
        (( equal? inGridType 2 ) ;vertical hex
          (script-fu-hex_grid img layerGrid 2 inGridSize 1 1 0 inGridSize inGridColour)
        )
      )    
    ) ; end of no grid check
    
    ;create wall shadows.
    (set! layerWallShadow (car (gimp-layer-new-from-drawable inLayer img)))
    (gimp-drawable-set-name layerWallShadow (string-append varLayerName " shadows"))
    (gimp-image-add-layer img layerWallShadow -1)   
    (gimp-layer-add-alpha layerWallShadow)
    (gimp-context-set-foreground (list 0 0 0))
    (gimp-drawable-fill layerWallShadow FOREGROUND-FILL)
    (gimp-context-set-foreground (list 255 255 255))
    (gimp-selection-load selInside)
    (gimp-selection-shrink img (/ inGridSize 8))
    (if (= inWallStyle 1) ;adjust if stroked
      (cond
        ((= inStrokePos 0) (gimp-selection-shrink img (/ inStrokeSize 2))) ;centered
        ((= inStrokePos 2) (gimp-selection-shrink img inStrokeSize)) ;inside
      )
    )
    (gimp-edit-bucket-fill layerWallShadow FG-BUCKET-FILL NORMAL-MODE 100 0 FALSE 0 0)
    (gimp-selection-none img)
    (plug-in-gauss RUN-NONINTERACTIVE img layerWallShadow (/ inGridSize 3) (/ inGridSize 3) 1)
    (gimp-selection-load selInside)
    (gimp-selection-invert img)
    (gimp-edit-bucket-fill layerWallShadow FG-BUCKET-FILL NORMAL-MODE 100 0 FALSE 0 0)
    (gimp-selection-none img)
    (plug-in-colortoalpha RUN-NONINTERACTIVE img layerWallShadow (list 255 255 255))
    (gimp-layer-set-opacity layerWallShadow 80)
    
    ;create filled wall layer
    (set! layerWall (car (gimp-layer-new-from-drawable inLayer img)))
    (gimp-drawable-set-name layerWall (string-append varLayerName " wall"))
    (gimp-image-add-layer img layerWall -1) 
    (gimp-layer-add-alpha layerWall)
    
    (if (= inWallMode 0)
      (begin   ;render floor
        (plug-in-plasma RUN-NONINTERACTIVE img layerWall (rand 32768) 7)
        (gimp-desaturate layerWall)
        (gimp-levels layerWall HISTOGRAM-VALUE 0 255 1 38 88)
        (plug-in-bump-map RUN-NONINTERACTIVE img layerWall layerWall inLightAngle 80 30 0 0 0 0 FALSE FALSE 0)
        ;cracks
        (set! layerTemp1 (car (gimp-layer-new-from-drawable inLayer img)))  ;create bump map
        (gimp-image-add-layer img layerTemp1 -1)    
        (plug-in-solid-noise RUN-NONINTERACTIVE img layerTemp1 FALSE TRUE (rand 32768) 15 15 15)
        (set! layerTemp2 (car (gimp-layer-new-from-drawable inLayer img)))  ;difference clouds
        (gimp-image-add-layer img layerTemp2 -1)    
        (plug-in-solid-noise RUN-NONINTERACTIVE img layerTemp2 FALSE TRUE (rand 32768) 15 15 15)
        (gimp-layer-set-mode layerTemp2 DIFFERENCE-MODE)    
        (set! layerTemp1 (car (gimp-image-merge-down img layerTemp2 EXPAND-AS-NECESSARY)))
        
        (set! layerTemp2 (car (gimp-layer-new-from-drawable inLayer img)))  ;difference clouds
        (gimp-image-add-layer img layerTemp2 -1)    
        (plug-in-solid-noise RUN-NONINTERACTIVE img layerTemp2 FALSE TRUE (rand 32768) 15 15 15)
        (gimp-layer-set-mode layerTemp2 DIFFERENCE-MODE)    
        (set! layerTemp1 (car (gimp-image-merge-down img layerTemp2 EXPAND-AS-NECESSARY)))
    
        (gimp-levels layerTemp1 HISTOGRAM-VALUE 0 40 0.1 0 255)     ;semi threshold
            
        ;bumpmap cracks
        (plug-in-bump-map RUN-NONINTERACTIVE img layerWall layerTemp1 inLightAngle 40 2 0 0 0 0 TRUE FALSE 3)       
        (gimp-image-remove-layer img layerTemp1) ;delete layer

      ) ;else
      (begin   ;use pattern
        (gimp-context-set-pattern inWallPattern)
        (gimp-drawable-fill layerWall 4) ;pattern fill = 4
      )
    )
    (gimp-selection-load selInside)  ;create a layer mask based on the selection
    (gimp-selection-invert img)
    (set! maskWall (car (gimp-layer-create-mask layerWall ADD-SELECTION-MASK)))
    (gimp-layer-add-mask layerWall maskWall)
    (gimp-selection-none img)
       
    ;wall styling
    (cond
      ((= inWallStyle 0) (begin ;beveled
        (set! layerTemp1 (car (gimp-layer-new-from-drawable inLayer img)))  ;create bump map
        (gimp-image-add-layer img layerTemp1 -1)    
        (gimp-context-set-foreground (list 255 255 255))
        (gimp-drawable-fill layerTemp1 FOREGROUND-FILL)
        (gimp-selection-load selInside)
        (gimp-context-set-foreground (list 0 0 0))
        (gimp-edit-bucket-fill layerTemp1 FG-BUCKET-FILL NORMAL-MODE 100 0 FALSE 0 0)
        (gimp-selection-none img)
        (plug-in-gauss RUN-NONINTERACTIVE img layerTemp1 (/ inGridSize 8) (/ inGridSize 8) 1)
        
        (set! layerWallStyle (car (gimp-layer-new-from-drawable inLayer img))) ;create bevel layer
        (gimp-drawable-set-name layerWallStyle (string-append varLayerName " wall style"))
        (gimp-image-add-layer img layerWallStyle -1)    
        (gimp-layer-add-alpha layerWallStyle)
        (gimp-context-set-foreground (list 127 127 127))
        (gimp-drawable-fill layerWallStyle FOREGROUND-FILL)
        (plug-in-bump-map RUN-NONINTERACTIVE img layerWallStyle layerTemp1 inLightAngle 10 10 0 0 0 0 TRUE FALSE 0)
        (gimp-layer-set-mode layerWallStyle OVERLAY-MODE)
        
        (gimp-image-remove-layer img layerTemp1) ;delete layer
        
        (gimp-selection-load selInside)  ;create a mask based on the selection
        (gimp-selection-invert img)
        (set! maskWall (car (gimp-layer-create-mask layerWallStyle ADD-SELECTION-MASK)))
        (gimp-layer-add-mask layerWallStyle maskWall)
        (gimp-selection-none img)
        
      ))

      ((= inWallStyle 1) (begin ;stroked
        (set! layerWallStyle (car (gimp-layer-new-from-drawable inLayer img))) ;create stroke layer
        (gimp-drawable-set-name layerWallStyle (string-append varLayerName " wall style"))
        (gimp-image-add-layer img layerWallStyle -1)    
        (gimp-layer-add-alpha layerWallStyle)
        (gimp-edit-clear layerWallStyle)
        
        (gimp-context-set-foreground inStrokeColour)

        (gimp-selection-load selInside)  ;load up selection
        
        (cond
          ((= inStrokePos 0)   ;centered
            (gimp-selection-grow img (/ inStrokeSize 2))
            (gimp-edit-fill layerWallStyle FG-BUCKET-FILL)
            (gimp-selection-load selInside)  ;load up selection
            (gimp-selection-shrink img (/ inStrokeSize 2))
            (gimp-edit-clear layerWallStyle) ;delete inside 
          )
          ((= inStrokePos 1)  ; outside
            (gimp-selection-grow img inStrokeSize)
            (gimp-edit-fill layerWallStyle FG-BUCKET-FILL)
            (gimp-selection-load selInside)  ;load up selection
            (gimp-edit-clear layerWallStyle) ;delete inside 
          )
          ((= inStrokePos 2)  ;inside
            (gimp-edit-fill layerWallStyle FG-BUCKET-FILL)
            (gimp-selection-shrink img inStrokeSize)
            (gimp-edit-clear layerWallStyle) ;delete inside 
          )
        )         
       
       (gimp-selection-none img)    
            
        (if (= inStrokeStyle 1) ; pattern filled
          (begin
            (gimp-selection-layer-alpha layerWallStyle)
            (gimp-context-set-pattern inStrokePattern)
            (gimp-edit-fill layerWallStyle 4) ;pattern fill = 4         
            (gimp-selection-none img)   
          )
        )
        
        (if (= inStrokeBevel TRUE)   ;bevel the stroke
          (begin
            (set! layerTemp1 (car (gimp-layer-new-from-drawable inLayer img)))  ;create bevel layer
            (gimp-image-add-layer img layerTemp1 -1)    
            (gimp-context-set-foreground (list 127 127 127))
            (gimp-drawable-fill layerTemp1 FOREGROUND-FILL)

            (set! layerTemp2 (car (gimp-layer-new-from-drawable inLayer img)))  ;create bump map
            (gimp-image-add-layer img layerTemp2 -1)    
            (gimp-context-set-foreground (list 0 0 0))
            (gimp-drawable-fill layerTemp2 FOREGROUND-FILL)
            (gimp-selection-layer-alpha layerWallStyle)    
            (gimp-context-set-foreground (list 255 255 255))
            (gimp-edit-fill layerTemp2 FG-BUCKET-FILL)
            (gimp-selection-none img)
            (plug-in-gauss RUN-NONINTERACTIVE img layerTemp2 (/ inStrokeSize 2) (/ inStrokeSize 2) 1)
            
            (plug-in-bump-map RUN-NONINTERACTIVE img layerTemp1 layerTemp2 inLightAngle 10 10 0 0 0 0 TRUE FALSE 0)
            (gimp-layer-set-mode layerTemp1 OVERLAY-MODE)
        
            (gimp-image-remove-layer img layerTemp2) ;delete layer
            (set! layerWallStyle (car (gimp-image-merge-down img layerTemp1 EXPAND-AS-NECESSARY)))  
            
          )
        )
        
      ))
    )     
    
    ;wall fade
    (when (or (= inWallFade 1) (= inWallFade 2)) ;black or white
      (set! layerWallFade (car (gimp-layer-new-from-drawable inLayer img)))
      (gimp-drawable-set-name layerWallFade (string-append varLayerName " wall fade"))
      (gimp-image-add-layer img layerWallFade -1) 
      (gimp-layer-add-alpha layerWallFade)

      (if (= inWallFade 1) (gimp-context-set-foreground (list 0 0 0)))
      (if (= inWallFade 2) (gimp-context-set-foreground (list 255 255 255)))
      
      (gimp-drawable-fill layerWallFade FOREGROUND-FILL)   
      (gimp-selection-load selInside)
      (gimp-selection-grow img (/ inGridSize 2))
      (gimp-selection-feather img (/ inGridSize 1.1))
      (gimp-edit-clear layerWallFade)
      (gimp-selection-none img)
    )
    
    (when (= inWallFade 3) ;trans - erase from other layers
      (gimp-context-set-foreground (list 0 0 0))     
      (gimp-selection-load selInside)
      (gimp-selection-grow img (/ inGridSize 2))
      (gimp-selection-feather img (/ inGridSize 1.1))
      (gimp-selection-invert img)
      (if (> layerFloor 0) (gimp-edit-clear layerFloor))
      (if (> layerFloorOverlay 0) (gimp-edit-clear layerFloorOverlay))
      (if (> layerWall 0) (gimp-edit-clear layerWall))
      (if (> layerGrid 0) (gimp-edit-clear layerGrid))
      (if (> layerGridBump 0) (gimp-edit-clear layerGridBump))
      (if (> layerWallStyle 0) (gimp-edit-clear layerWallStyle))
      (if (> layerWallShadow 0) (gimp-edit-clear layerWallShadow))
      (gimp-selection-none img)
    )
    
    ;delete distorted layer,
    (if (> inDistort 0)
      (gimp-image-remove-layer img inLayer) ;delete layer     
    )  
    ;remove the selection channel
    (gimp-image-remove-channel img selInside)
    
    ; crop the image back down
    (gimp-image-crop img (- width (* 2 inGridSize)) (- height (* 2 inGridSize)) inGridSize inGridSize)
    
    ;set active layer
    (if (= layerFloor 0)
      (gimp-image-set-active-layer img origlayer)
      (gimp-image-set-active-layer img layerFloor))      
    
    ;turn off template visibility
    (gimp-drawable-set-visible origlayer FALSE)    
     
    ;done
    (gimp-image-undo-group-end img)
    (gimp-displays-flush)
    (gimp-progress-end)
    (gimp-context-pop)
  )
)

(script-fu-register "script_fu_dungeon_map_maker"
                    "<Image>/Filters/Dungeon Map Maker..."
                    "Create an artistic dungeon map from a template"
                    "Rob Antonishen"
                    "Rob Antonishen"
                    "August 2008"
                    "RGB* GRAY*"
                    SF-IMAGE      "image"      0
                    SF-DRAWABLE   "drawable"   0
                    SF-ADJUSTMENT "Grid Size" (list 50 10 200 5 10 0 SF-SLIDER)
                    SF-OPTION     "Grid Type" (list "Square" "Horizontal Hex" "Vertical Hex" "None")
                    SF-COLOR      "Template Colour?"  (list 255 255 255)
                    SF-ADJUSTMENT "Colour Selection Threshold" (list 10 0 255 1 10 0 SF-SLIDER)                 
                    SF-OPTION     "Template Colour Identifies"   (list "Floors" "Walls")
                    SF-ADJUSTMENT "Distort Amount" (list 0 0 100 1 25 0 SF-SLIDER)
                    SF-CHANNEL    "Distortion Mask" -1
                    SF-ADJUSTMENT "Lighting Azimuth" (list 135 0 360 1 15 2 SF-SLIDER)
                    SF-OPTION     "Fill Floor With"   (list "Rendered Noise" "Pattern - Select Below" "No Floor")
                    SF-PATTERN    "Floor Pattern"    "Dried mud"
                    SF-TOGGLE     "Add Floor Noise Overlay?" TRUE                 
                    SF-COLOR      "Grid Colour?"  (list 0 0 0)
                    SF-OPTION     "Fill Walls With"   (list "Rendered Noise" "Pattern - Select Below")
                    SF-PATTERN    "Wall Pattern"    "Crack"
                    SF-OPTION     "Wall Style"   (list "Bevel" "Stroked" "None")
                    SF-OPTION     "Wall Fade" (list "None" "Black" "White" "Transparent")
                    SF-OPTION     "Stroke Style"   (list "Colour" "Pattern")
                    SF-TOGGLE     "Bevel Stroke?" FALSE                 
                    SF-ADJUSTMENT "Stroke Size" (list 5 1 50 1 5 0 SF-SLIDER)
                    SF-OPTION     "Stroke Position"   (list "Centered" "Outside" "Inside" )
                    SF-COLOR      "Stroke Colour?"  (list 128 128 128)
                    SF-PATTERN    "Stroke Pattern"    "Fibers"
)

(define (script_fu_dungeon_map_batch inDir inLoadType inSaveType inKeepTemp inPostfix
                                     inGridSize inGridType inSelectColour inSelectThresh inSelectWhat inDistort inLightAngle
                                     inFloorMode inFloorPattern inFloorNoise inGridColour
                                     inWallMode inWallPattern inWallStyle inWallFade
                                     inStrokeStyle inStrokeBevel inStrokeSize inStrokePos inStrokeColour inStrokePattern)
                                     
  (let*
    (
      (varLoadStr "")
      (varSaveStr "")
      (varFileList 0)
      (handler (car (gimp-message-get-handler)))
    )
    ;begin here
    (set! varLoadStr
      (cond 
        (( equal? inLoadType 0 ) ".jpg" )
        (( equal? inLoadType 1 ) ".bmp" )
        (( equal? inLoadType 2 ) ".png" )
        (( equal? inLoadType 3 ) ".gif" )
      )
    )  
  
    (set! varSaveStr
      (cond 
        (( equal? inLoadType 0 ) ".jpg" )
        (( equal? inLoadType 1 ) ".bmp" )
        (( equal? inLoadType 2 ) ".png" )
        (( equal? inLoadType 3 ) ".gif" )
      )
    )

    ;check for hex option and no script
    (when (and (or (= inGridType 1) (= inGridType 2)) (not (defined? 'script-fu-hex_grid)))
      (gimp-message-set-handler MESSAGE-BOX)
      (gimp-message "Hex Grid Script must be installed.\nIt can be downloaded from\nhttp://registry.gimp.org/node/15852")
      (gimp-message-set-handler handler)
      (quit)
    )
    
    (set! varFileList (cadr (file-glob (string-append inDir DIR-SEPARATOR "*" varLoadStr)  1)))
    
    (while (not (null? varFileList))
      (let* ((filename (car varFileList))
             (image (car (gimp-file-load RUN-NONINTERACTIVE filename filename)))
             (drawable (car (gimp-image-get-active-layer image)))
             (newfilename (string-append (substring filename 0 (- (string-length filename) 4)) inPostfix varSaveStr))
             (xcffilename (string-append (substring filename 0 (- (string-length filename) 4)) ".xcf"))
            )
          
;        (gimp-message (string-append "Working on ->" filename))
  
         (script_fu_dungeon_map_maker image drawable
                                     inGridSize inGridType inSelectColour inSelectThresh inSelectWhat inDistort -1 inLightAngle
                                     inFloorMode inFloorPattern inFloorNoise inGridColour
                                     inWallMode inWallPattern inWallStyle inWallFade
                                     inStrokeStyle inStrokeBevel inStrokeSize inStrokePos inStrokeColour inStrokePattern)
      
          (if (= inKeepTemp TRUE)
            (gimp-file-save RUN-NONINTERACTIVE image (car (gimp-image-get-active-layer image)) xcffilename xcffilename)
          )
          
          (set! drawable (car (gimp-image-flatten image)))

          (gimp-file-save RUN-NONINTERACTIVE image drawable newfilename newfilename)

          (gimp-image-delete image)
      )
      (set! varFileList (cdr varFileList))
    )
  )  
)
  
(script-fu-register "script_fu_dungeon_map_batch"
                    "<Toolbox>/Xtns/Batch Tools/Dungeon Map Batch Processor..."
                    "Create Dungeon Maps for all files in a directory"
                    "Rob Antonishen"
                    "Rob Antonishen"
                    "August 2008"
                    ""
                    SF-DIRNAME    "Load from" ""
                    SF-OPTION     "Load File Type" (list "jpg" "bmp" "png" "gif")
                    SF-OPTION     "Save File Type" (list "jpg" "bmp" "png" "gif")
                    SF-TOGGLE     "Keep xcf file?" TRUE                         
                    SF-STRING     "Save Postfix"     "-DMM"
                    SF-ADJUSTMENT "Grid Size" (list 50 10 200 5 10 0 SF-SLIDER)
                    SF-OPTION     "Grid Type" (list "Square" "Horizontal Hex" "Vertical Hex" "None")
                    SF-COLOR      "Template Colour?"  (list 255 255 255)
                    SF-ADJUSTMENT "Colour Selection Threshold" (list 10 0 255 1 10 0 SF-SLIDER)                 
                    SF-OPTION     "Template Colour Identifies"   (list "Floors" "Walls")
                    SF-ADJUSTMENT "Distort Amount" (list 0 0 100 1 25 0 SF-SLIDER)
                    SF-ADJUSTMENT "Lighting Azimuth" (list 135 0 360 1 15 2 SF-SLIDER)
                    SF-OPTION     "Fill Floor With"   (list "Rendered Noise" "Pattern - Select Below" "No Floor")
                    SF-PATTERN    "Floor Pattern"    "Dried mud"
                    SF-TOGGLE     "Add Floor Noise Overlay?" TRUE                 
                    SF-COLOR      "Grid Colour?"  (list 0 0 0)
                    SF-OPTION     "Fill Walls With"   (list "Rendered Noise" "Pattern - Select Below")
                    SF-PATTERN    "Wall Pattern"    "Crack"
                    SF-OPTION     "Wall Style"   (list "Bevel" "Stroked" "None")
                    SF-OPTION     "Wall Fade" (list "None" "Black" "White" "Transparent")
                    SF-OPTION     "Stroke Style"   (list "Colour" "Pattern")
                    SF-TOGGLE     "Bevel Stroke?" FALSE                 
                    SF-ADJUSTMENT "Stroke Size" (list 5 1 50 1 5 0 SF-SLIDER)
                    SF-OPTION     "Stroke Position"   (list "Centered" "Outside" "Inside" )
                    SF-COLOR      "Stroke Colour?"  (list 128 128 128)
                    SF-PATTERN    "Stroke Pattern"    "Fibers"
)